/*
 * (POPUP WINDOW)
 * main.js - Botize Scraper
 * https://botize.com/es/app/btzscraper
 * 
 */

let scraper_tab_id; // Browser tab Id where access_token was setted last time and where scrapper is working on
let access_token;   // Botize Account Access token

load_data();

function load_data() {

    chrome.storage.local.get(["scraper_tab_id","access_token"]).then((result) => {

        scraper_tab_id = result.scraper_tab_id;
        access_token = result.access_token;

        initialize();
        
        console.log("Scraper is running on tab "+scraper_tab_id);
    });
}

setInterval(() => {

    text = txt();

    // Show Status code
    chrome.storage.local.get(["status_code","status_message"]).then((result) => {

        var elem = document.getElementById("status_code");

        if (elem===null)
            return;

        if (text[result.status_code])
            elem.innerHTML =text[result.status_code];
        else
            elem.innerHTML = "";
        
        // Show Status message
        document.getElementById("status_message").innerHTML =result.status_message;
    });

    
    
    
    
  }, 200);

function initialize() {

    chrome.tabs.query({active: true, currentWindow: true}, function(tabs) {
        var tab = tabs[0];

        var data = {
            action: "executeFunction",
            formData: {
                functionName: "getExtensionConfig"
            }
        };

        chrome.tabs.sendMessage(tab.id, data, function(raw_response) {
            
            if (raw_response==undefined)
                return;
                
            var response = JSON.parse(raw_response);

            response["scraper_tab_id"]=scraper_tab_id;
            response["access_token"]=access_token;
            response["title"]="BTZScraper";
            response["tab_id"]=tab.id;
            response["txt"]=txt();

            if (!response["scraper_tab_id"])
                response["scraper_not_connected"]=true;
            
            if (response["scraper_tab_id"] && tab.id != response["scraper_tab_id"]) {
                // Scraper is running but in a different tab
                response["running_in_a_different_tab"]=true;
            }

            if (response["scraper_tab_id"] && tab.id == response["scraper_tab_id"]) {
                // Scraper is running on this tab
                response["running_on_this_tab"]=true;
            }
            
            // popup HTML
            var template = `
            
            <h1>{{title}}</h1>

            {{#running_in_a_different_tab}}

                <p>{{txt.SCRAPER_WORKING_ON_ANOTHER_TAB}}</p>

                <form>
                    <input type="hidden" name="functionName" value="formRemoveAccessToken">
                    <input type="submit" value="{{txt.DISCONNECT}}">
                </form>

            {{/running_in_a_different_tab}}

            {{#running_on_this_tab}}
                <p id="status_code" class="status-code"></p>
                <p id="status_message" class="status-message">Status message...</p>
            {{/running_on_this_tab}}

            {{#scraper_not_connected}}
        
                <form>
                    <label for="access_token">Access Token</label>
                    <input type="text" id="access_token" name="access_token">
                    <input type="hidden" name="functionName" value="formSetAccessToken">
                    <br><br>
                    <input type="submit" value="{{txt.CONNECT}}">
                </form>

            {{/scraper_not_connected}}

            {{#running_on_this_tab}}
            
                <form>
                    <input type="hidden" name="functionName" value="formRemoveAccessToken">
                    <input type="submit" value="{{txt.DISCONNECT}}">
                </form>

                <!--
                <form>
                    <input type="hidden" name="functionName" value="formResetJobs">
                    <input type="submit" value="Reset">
                </form>
                -->

                {{#meta.error_message}}
                    <span style="color:red">{{.}}</span>
                {{/meta.error_message}}

                <!--<hr><div id='debugger'></div>-->
                
            {{/running_on_this_tab}}
            
            `;

            document.body.innerHTML = Mustache.render(template, response);
            
            const formElements = document.querySelectorAll("form");

            formElements.forEach(element => {
                element.addEventListener('submit', handleFormSubmit);
            });
            
        });

    });
}

function handleFormSubmit(event) {
    
    event.returnValue = false; // Prevents default form submission behavior

    chrome.tabs.query({active: true, currentWindow: true}, function(tabs) {

        var tab = tabs[0];

        const formData = new FormData(event.target);

        const dataDict = {};
  
        for (const [key, value] of formData.entries()) {
            dataDict[key] = value;
        }
        
        var data = {
            action: "executeFunction",
            formData: dataDict
        };

        chrome.tabs.sendMessage(tab.id, data, function(response) {
            load_data();
        });

    });
}

function txt() {
    
    let language = navigator.language;

    var texts = {
        "es": {
            "DISCONNECT":"Desconectar",
            "CONNECT":"Conectar",
            "SCRAPER_WORKING_ON_ANOTHER_TAB":"El scraper se encuentra en uso en otra pestaña del navegador. Pulsa el botón 'Desconectar' para liberarlo y permitir su uso en esta pestaña.",
            "SCRAPER_WORKING_ON_THIS_TAB":"Scraper en uso en este pestaña",
            "STATUS_WAITING_JOB":"✓ Ready for scraping",
            "STATUS_PROCESSING_JOB":"Scraping..."
        },
        "en": {
            "DISCONNECT":"Disconnect",
            "CONNECT":"Connect",
            "SCRAPER_WORKING_ON_ANOTHER_TAB":"The scraper is in use on another browser tab. Click the 'Disconnect' button to release it and allow its use in this tab.",
            "SCRAPER_WORKING_ON_THIS_TAB":"Scraper in use on this tab",
            "STATUS_WAITING_JOB":"✓ Ready for scraping",
            "STATUS_PROCESSING_JOB":"Scraping..."
        }
    }

    if (language.startsWith("es"))
        return texts["es"];
    
    return texts["en"];
    
}