/*!
 * script.js - Botize Scraper
 * https://botize.com/es/app/btzscraper
 * 
 * Note: This script runs whenever the browser loads a new URL.
 */

let tab_id;         // Browser tab Id where the extension is running
let scraper_tab_id; // Browser tab Id where access_token was setted last time and where scrapper is working on
let access_token;   // Botize Account Access token
let wait_timestamp; // Keeps the scraper in waiting mode until the indicated timestamp is exceeded
let url_counter;    // How many times a new URL is loaded
let job;            // Current job or undefined
let step;           // Current step or undefined
let blob_url;       // File in blob

console.log("Start.");

chrome.storage.local.get(["url_counter","access_token"], function (result) {

	url_counter = result.url_counter || 0;
	url_counter += 1;

	chrome.storage.local.set({
			"url_counter": url_counter
		}).then(() => {
			console.log("URL COUNTER SETTED!");
	});

	console.log("URL COUNTER: "+url_counter);
});

chrome.runtime.sendMessage({ "action":"get_my_tab_id" }, function (response) {
	console.log(response);
	tab_id = response["tab_id"];
});

document.addEventListener('click', function(event) {
	var xpath = getXPath(event.target);
	console.log(xpath);
});

function getXPathOfFocusedElement() {

	var focusedElement = document.activeElement;

	if (focusedElement && focusedElement.tagName !== 'BODY') {

		var xpath = '';
		
		while (focusedElement !== document.body) {
			var tagName = focusedElement.tagName.toLowerCase();
			var index = 1;
			var sibling = focusedElement.previousSibling;
			
			while (sibling) {
				if (sibling.tagName && sibling.tagName.toLowerCase() === tagName) {
					index++;
				}
				sibling = sibling.previousSibling;
			}

			xpath = '/' + tagName + '[' + index + ']' + xpath;
			focusedElement = focusedElement.parentNode;
		}
		return xpath;
	}
	return null;
}

function getXPath(element) {
  var xpath = '';
  while (element !== document.documentElement) {
	var siblingIndex = 1;
	var sibling = element.previousElementSibling;
	while (sibling) {
	  if (sibling.nodeName === element.nodeName) {
		siblingIndex++;
	  }
	  sibling = sibling.previousElementSibling;
	}
	var tagName = element.nodeName.toLowerCase();
	var id = element.getAttribute('id');
	var className = element.getAttribute('class');
	var index = siblingIndex > 1 ? '[' + siblingIndex + ']' : '';
	var identifier = '';
	if (id) {
	  identifier = '#' + id;
	} else if (className) {
	  identifier = '.' + className.replace(/\s+/g, '.');
	}
	xpath = '/' + tagName + identifier + index + xpath;
	element = element.parentNode;
  }
  xpath = '//' + xpath;
  return xpath;
}

  setInterval(() => {
  
	try{
	  chrome.storage.local.get(["scraper_tab_id","access_token","wait_timestamp","job","step"]).then((result) => {

		scraper_tab_id = result.scraper_tab_id;
		access_token = result.access_token;
		wait_timestamp = result.wait_timestamp;

		job = result.job; // undefined if not exists or JOB array if exists
		step = result.step; // undefined if not exists or STEP array if exists

		start();

	  }).catch((error) => {
		  console.error("Refresh this page. ", error);
	  });
	}
	catch(error) {
	  console.error("Refresh this page. ", error);
	}
	  
  }, 1000);


function start() {

  path = getXPathOfFocusedElement();

  if (!access_token) {
	console.log("MAIN: Extension disabled. Access token required");
	return;
  }

  if (tab_id != scraper_tab_id) {
	console.log("MAIN: Scraper is working on tab Id "+scraper_tab_id+" but this tab is "+tab_id);
	return;
  }

  if (Date.now()<wait_timestamp) {
	
	var elapse = (wait_timestamp - Date.now())/1000;
	//console.log("MAIN: Waiting "+elapse+" seconds");

	if (elapse>60) {
	  // Maximum stablished to 60 seconds
	  chrome.storage.local.set({
		"wait_timestamp":Date.now()+60*1000
	  });
	}

	chrome.storage.local.set({
	  "status_message":Math.round(elapse)+" seconds remaining"
	});

	return;
  }

  // Check for a step in memory
  
  if (step !== undefined) {
	// Currently script is working with a step -> Wait
	console.log("MAIN: Current step: ");
	console.log(step);

	let elapsedTimeInSeconds = (Date.now() - step["start_at"]) / 1000;

	if (elapsedTimeInSeconds>30) {
		console.log("MAIN: Error. The wait time has exceeded 30 seconds ("+elapsedTimeInSeconds+"s). The current step of the job is considered failed, and the entire job will now be forcibly terminated");
		chrome.storage.local.remove(["step"]);
	}
	else
		console.log("MAIN: Waiting "+elapsedTimeInSeconds+" seconds for the step to finish");

	return;
  }

  // Check for a job in memory
  if (job === undefined) {
	
	stepFunctionType_search_job();
	return;
  }
  else {

	// There's a job in memory -> Get next step
	console.log("MAIN: JOB -> "+job);
	getNextStep();
  }

  if (step !== undefined) {

	try {
	  processStep();
	} catch (error) {

	  console.error('Error: ', error.message);
	  cancelJob(error.message);
	}
	
  }
}

  function getNextStep() {
	
	// Get Job
	// job = JSON.parse(localStorage.getItem("job"));

	if (job["steps"].length==0) {
	  console.log("GetNextStep(): No more steps -> Remove Job");

	  // localStorage.removeItem("job");
	  
	  chrome.storage.local.remove(["job"]);
	  return;
	}

	// Remove first step from job["steps"] and add it var step
	step = job["steps"].shift();
	
	step["start_at"] = Date.now();

	// Save step in memory and update job
	chrome.storage.local.set({
	  "job":job,
	  "step":step
	});
  }

  // Start processing a job step
  function processStep() {
	
	// Get Job
	step["access_token"]=access_token;

	// Process step
	console.log("processStep(): Start processing JOB step: "+JSON.stringify(step));
	
	functionName="stepFunctionType_"+step["type"];

	if (typeof window[functionName] === "function") {

	  // Call function
	  var response = window[functionName](step);
	
	} else {
	  console.log("processStep(): Unknown functionName: '"+functionName+"'. Make sure you have included the corresponding JS file in the manifest.json file before proceeding");
	  //localStorage.removeItem("step");
	  chrome.storage.local.remove(["step"]);
	}

	if (response)
	  console.log("processStep(): Step '"+functionName+"' returns:"+response);

  }
 
  function cancelJob(error_message) {
	//alert("Job from "+job["node"]+" with Id "+job["mb_run_id"]+" has been canceled");

	var step = {
		"node":job["node"],
		"mb_run_id":job["mb_run_id"],
		"access_token":access_token,
		"error_message":error_message
	};

	stepFunctionType_job_error(step);
  }

  function loadJob(data) {

	console.log("Processing this job:");
	console.log(data);

	chrome.storage.local.set({"job":data});
	chrome.storage.local.remove(["step"]);
	
  }

function formSetAccessToken(callback,payload){

	if (payload["formData"]["access_token"]!="") {
	  
	  updateConfig("meta",{"message":"Access token setted"});
	  
	  chrome.storage.local.set({
		"access_token":payload["formData"]["access_token"],
		"scraper_tab_id":tab_id
	  }).then(() => {
		//console.log("formSetAccessToken(): Setted access_token to "+payload["formData"]["access_token"]);
	  });
	  
	  callback(true);
	  return;
	}
	
	console.log("formSetAccessToken(): Can't sent an empty access token");
	updateConfig("meta",{"error_message":"Invalid access token"});
	callback(false);
	
  }

  function formRemoveAccessToken(callback,payload){
	
	console.log("formRemoveAccessToken(): Remove access token and scraper_tab_id");

	chrome.storage.local.remove([
	  "access_token",     // Remove access token
	  "scraper_tab_id",   // No Tab Id for scraping
	],function(){

	  access_token=undefined;
	  scraper_tab_id=undefined;

	  //localStorage.removeItem("job");
	  // localStorage.removeItem("step");
	  chrome.storage.local.remove(["job","step"]);

	  console.log("formRemoveAccessToken(): removed!");
	  var error = chrome.runtime.lastError;
	
		 if (error) {
			console.log(error);
			updateConfig("meta",{"error_message":error});
		 }
		 else
			updateConfig("meta",{"message":"Access token has been removed"});
	 })

	callback(true);
	
  }

  function formResetJobs(callback,payload){
	
	console.log("clear memory");
	
	chrome.storage.local.remove(["job","step"]);

	updateConfig("meta",{"message":"Current job removed"});
	callback(true);
  }

  function updateConfig(key,value) {
	
	if (localStorage.getItem("extension_config") === null)
	  data = "{}"
	else
	  data =localStorage.getItem("extension_config");

	data = JSON.parse(data);
	data[key]=value;

	localStorage.setItem("extension_config",JSON.stringify(data));

  }

  function getFromConfig(key) {
	
	if (localStorage.getItem("extension_config") === null)
	  data = "{}"
	else
	  data =localStorage.getItem("extension_config");

	data = JSON.parse(data);

	return data[key];
  }

  function getExtensionConfig(callback){

	if (localStorage.getItem("extension_config") === null)
	  callback('{"empty":"super empty"}');

	callback(localStorage.getItem("extension_config"));
  }

  function stop_extension(callback){
	console.log("you ask me to stop the extension");
	localStorage.setItem("running",false);
	callback(true);
  }

  function start_extension(callback){
	console.log("you ask me to start the extension");
	localStorage.setItem("running",true);
	callback(true);
  }

  // Receive callbacks from main.js and process it
  chrome.runtime.onMessage.addListener(
	function(request, sender, sendResponse) {
	  if(request.action == "executeFunction"){
		functionName = request.formData["functionName"];
		window[functionName](sendResponse,request);
	  }
	  return true;
	}
  );
